@@ -305,9 +305,7 @@ palisade_start (
 	/*
 	 * Allocate and initialize unit structure
 	 */
-	up = (struct palisade_unit *) emalloc(sizeof(struct palisade_unit));
-
-	memset((char *)up, 0, sizeof(struct palisade_unit));
+	up = emalloc_zero(sizeof(*up));
 
 	up->type = CLK_TYPE(peer);
 	switch (up->type) {
@@ -315,20 +313,20 @@ palisade_start (
 		/* Normal mode, do nothing */
 		break;
 	    case CLK_PRAECIS:
-		msyslog(LOG_NOTICE, "Palisade(%d) Praecis mode enabled\n"
+		msyslog(LOG_NOTICE, "Palisade(%d) Praecis mode enabled"
 			,unit);
 		break;
 	    case CLK_THUNDERBOLT:
-		msyslog(LOG_NOTICE, "Palisade(%d) Thunderbolt mode enabled\n"
+		msyslog(LOG_NOTICE, "Palisade(%d) Thunderbolt mode enabled"
 			,unit);
 		tio.c_cflag = (CS8|CLOCAL|CREAD);
 		break;
 	    case CLK_ACUTIME:
-		msyslog(LOG_NOTICE, "Palisade(%d) Acutime Gold mode enabled\n"
+		msyslog(LOG_NOTICE, "Palisade(%d) Acutime Gold mode enabled"
 			,unit);
 		break;
 	    default:
-		msyslog(LOG_NOTICE, "Palisade(%d) mode unknown\n",unit);
+		msyslog(LOG_NOTICE, "Palisade(%d) mode unknown",unit);
 		break;
 	}
 	if (tcsetattr(fd, TCSANOW, &tio) == -1) {
@@ -343,7 +341,7 @@ palisade_start (
 
 	pp = peer->procptr;
 	pp->io.clock_recv = palisade_io;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
@@ -359,7 +357,7 @@ palisade_start (
 	/*
 	 * Initialize miscellaneous variables
 	 */
-	pp->unitptr = (caddr_t)up;
+	pp->unitptr = up;
 	pp->clockdesc = DESCRIPTION;
 
 	peer->precision = PRECISION;
@@ -394,7 +392,7 @@ palisade_shutdown (
 	struct palisade_unit *up;
 	struct refclockproc *pp;
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 	if (-1 != pp->io.fd)
 		io_closeclock(&pp->io);
 	if (NULL != up)
@@ -448,7 +446,7 @@ TSIP_decode (
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Check the time packet, decode its contents. 
@@ -544,7 +542,7 @@ TSIP_decode (
 
 #ifdef DEBUG
 			if (debug > 1)
-				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%06ld %02d/%02d/%04d UTC %02d\n",
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d UTC %02d\n",
 				       up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, 
 				       pp->second, pp->nsec, mb(12), mb(11), pp->year, GPS_UTC_Offset);
 #endif
@@ -586,7 +584,10 @@ TSIP_decode (
 				break;
 			}
 
-			if (up->leap_status & PALISADE_LEAP_PENDING) {
+			up->month = mb(15);
+			if ( (up->leap_status & PALISADE_LEAP_PENDING) &&
+			/* Avoid early announce: https://bugs.ntp.org/2773 */
+				(6 == up->month || 12 == up->month) ) {
 				if (up->leap_status & PALISADE_UTC_TIME)  
 					pp->leap = LEAP_ADDSECOND;
 				else
@@ -617,10 +618,11 @@ TSIP_decode (
 			pp->hour = mb(11);
 			pp->minute = mb(12);
 			pp->second = mb(13);
+			up->month = mb(14);  /* Save for LEAP check */
 
 #ifdef DEBUG
 			if (debug > 1)
-				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%06ld %02d/%02d/%04d UTC %02x %s\n",
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d UTC %02x %s\n",
 				       up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, 
 				       pp->second, pp->nsec, mb(15), mb(14), pp->year,
 				       mb(19), *Tracking_Status[st]);
@@ -647,7 +649,9 @@ TSIP_decode (
 				printf("TSIP_decode: unit %d\n", up->unit);
 			}
 #endif
-			if (getint((u_char *) &mb(10)) & 0x80) 
+			if ( (getint((u_char *) &mb(10)) & 0x80) &&
+			/* Avoid early announce: https://bugs.ntp.org/2773 */
+			    (6 == up->month || 12 == up->month) )
 				pp->leap = LEAP_ADDSECOND;  /* we ASSUME addsecond */
 			else 
 				pp->leap = LEAP_NOWARNING;
@@ -764,7 +768,7 @@ TSIP_decode (
 
 #ifdef DEBUG
 			if (debug > 1)
-				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%06ld %02d/%02d/%04d ",up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, pp->second, pp->nsec, mb(14), mb(13), pp->year);
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d ",up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, pp->second, pp->nsec, mb(14), mb(13), pp->year);
 #endif
 			return 1;
 			break;
@@ -835,15 +839,15 @@ TSIP_decode (
 			up->polled = -1;
 #ifdef DEBUG
 			if (debug > 1) {
-				if (mb(1) && 0x01)
+				if (mb(1) & 0x01)
 					printf ("Signal Processor Error, reset unit.\n");
-				if (mb(1) && 0x02)
+				if (mb(1) & 0x02)
 					printf ("Alignment error, channel or chip 1, reset unit.\n");
-				if (mb(1) && 0x03)
+				if (mb(1) & 0x03)
 					printf ("Alignment error, channel or chip 2, reset unit.\n");
-				if (mb(1) && 0x04)
+				if (mb(1) & 0x04)
 					printf ("Antenna feed line fault (open or short)\n");
-				if (mb(1) && 0x05)
+				if (mb(1) & 0x05)
 					printf ("Excessive reference frequency error, refer to packet 0x2D and packet 0x4D documentation for further information\n");
 			}
 #endif
@@ -900,7 +904,7 @@ palisade_receive (
 	 * Initialize pointers and read the timecode and timestamp.
 	 */
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 		
 	if (! TSIP_decode(peer)) return;
 	
@@ -912,7 +916,7 @@ palisade_receive (
 #ifdef DEBUG
 	if (debug) 
 		printf(
-			"palisade_receive: unit %d: %4d %03d %02d:%02d:%02d.%06ld\n",
+			"palisade_receive: unit %d: %4d %03d %02d:%02d:%02d.%09ld\n",
 			up->unit, pp->year, pp->day, pp->hour, pp->minute, 
 			pp->second, pp->nsec);
 #endif
@@ -924,7 +928,7 @@ palisade_receive (
 	 */
 
 	snprintf(pp->a_lastcode, sizeof(pp->a_lastcode),
-		 "%4d %03d %02d:%02d:%02d.%06ld",
+		 "%4d %03d %02d:%02d:%02d.%09ld",
 		 pp->year, pp->day,
 		 pp->hour,pp->minute, pp->second, pp->nsec); 
 	pp->lencode = 24;
@@ -965,7 +969,7 @@ palisade_poll (
 	struct refclockproc *pp;
 	
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	pp->polls++;
 	if (up->polled > 0) /* last reply never arrived or error */ 
@@ -1038,9 +1042,9 @@ palisade_io (
 
 	char * c, * d;
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	if(up->type == CLK_PRAECIS) {
 		if(praecis_msg) {
@@ -1141,14 +1145,12 @@ HW_poll (
 	int x;	/* state before & after RTS set */
 	struct palisade_unit *up;
 
-	up = (struct palisade_unit *) pp->unitptr;
+	up = pp->unitptr;
 
 	/* read the current status, so we put things back right */
 	if (ioctl(pp->io.fd, TIOCMGET, &x) < 0) {
-#ifdef DEBUG
-		if (debug)
-			printf("Palisade HW_poll: unit %d: GET %s\n", up->unit, strerror(errno));
-#endif
+		DPRINTF(1, ("Palisade HW_poll: unit %d: GET %m\n",
+			up->unit));
 		msyslog(LOG_ERR, "Palisade(%d) HW_poll: ioctl(fd,GET): %m", 
 			up->unit);
 		return -1;
@@ -1232,7 +1234,10 @@ getint (
 	u_char *bp
 	)
 {
-	return (short)ntohs(*(u_short *)bp);
+	u_short us;
+
+	memcpy(&us, bp, sizeof(us));
+	return (short)ntohs(us);
 }
 
 /*
@@ -1243,7 +1248,10 @@ getlong(
 	u_char *bp
 	)
 {
-	return (int32)(u_int32)ntohl(*(u_int32 *)bp);
+	u_int32 u32;
+
+	memcpy(&u32, bp, sizeof(u32));
+	return (int32)(u_int32)ntohl(u32);
 }
 
 #else	/* REFCLOCK && CLOCK_PALISADE*/
